[PATCH] D103775: [lld/mac] Implement support for searching dylibs with @executable_path/ in install name

Nico Weber via Phabricator via llvm-commits llvm-commits at lists.llvm.org
Sun Jun 6 13:58:33 PDT 2021


thakis created this revision.
thakis added a reviewer: lld-macho.
Herald added a reviewer: int3.
Herald added a reviewer: gkm.
Herald added a project: lld-macho.
thakis requested review of this revision.

https://reviews.llvm.org/D103775

Files:
  lld/MachO/InputFiles.cpp
  lld/test/MachO/link-search-at-executable-path.s


Index: lld/test/MachO/link-search-at-executable-path.s
===================================================================
--- /dev/null
+++ lld/test/MachO/link-search-at-executable-path.s
@@ -0,0 +1,41 @@
+# REQUIRES: x86
+
+# RUN: rm -rf %t; split-file %s %t
+# RUN: mkdir %t/subdir
+
+# RUN: llvm-mc -filetype obj -triple x86_64-apple-darwin %t/foo.s -o %t/foo.o
+# RUN: llvm-mc -filetype obj -triple x86_64-apple-darwin %t/bar.s -o %t/bar.o
+# RUN: llvm-mc -filetype obj -triple x86_64-apple-darwin %t/main.s -o %t/main.o
+
+# RUN: %lld -dylib -install_name @executable_path/libfoo.dylib %t/foo.o -o %t/subdir/libfoo.dylib
+
+# RUN: %lld -dylib -reexport_library %t/subdir/libfoo.dylib %t/bar.o -o %t/libbar.dylib
+
+## When linking executables, @executable_path/ in install_name should be replaced
+## by the path of the executable.
+# RUN: %lld -lSystem %t/main.o %t/libbar.dylib -o %t/subdir/test
+
+## This doesn't work for non-executables.
+# RUN: not %lld -dylib -lSystem %t/main.o %t/libbar.dylib -o %t/subdir/libtest.dylib 2>&1 | FileCheck --check-prefix=ERR %s
+
+## It also doesn't help if the needed reexport isn't next to the library.
+# RUN: not %lld -lSystem %t/main.o %t/libbar.dylib -o %t/test 2>&1 | FileCheck --check-prefix=ERR %s
+# ERR: error: unable to locate re-export with install name @executable_path/libfoo.dylib
+
+#--- foo.s
+.globl _foo
+_foo:
+  retq
+
+#--- bar.s
+.globl _bar
+_bar:
+  retq
+
+#--- main.s
+.section __TEXT,__text
+.global _main
+_main:
+  callq _foo
+  callq _bar
+  ret
Index: lld/MachO/InputFiles.cpp
===================================================================
--- lld/MachO/InputFiles.cpp
+++ lld/MachO/InputFiles.cpp
@@ -760,7 +760,15 @@
               resolveDylibPath((root + path).str()))
         return loadDylib(*dylibPath, umbrella);
 
-  // TODO: Expand @loader_path, @executable_path, @rpath etc, handle -dylib_path
+  // TODO: Expand @loader_path, @rpath etc, handle -dylib_file
+  SmallString<128> newPath;
+  if (config->outputType == MH_EXECUTE &&
+      path.consume_front("@executable_path/")) {
+    // ld64 allows overriding this with the undocumented flag -executable_path.
+    // lld doesn't currently implement that flag.
+    path::append(newPath, sys::path::parent_path(config->outputFile), path);
+    path = newPath;
+  }
 
   if (currentTopLevelTapi) {
     for (InterfaceFile &child :


-------------- next part --------------
A non-text attachment was scrubbed...
Name: D103775.350132.patch
Type: text/x-patch
Size: 2377 bytes
Desc: not available
URL: <http://lists.llvm.org/pipermail/llvm-commits/attachments/20210606/2339b6bc/attachment.bin>


More information about the llvm-commits mailing list